part of 'paging_widget.dart';

mixin PagingAction<Q, D, B extends BasePagingBloc<Q, D>,
    T extends StatefulWidget> on State<T> {
  void onFirstQuery(
    Q? queryData,
    int limit, {
    bool needLoading = true,
  }) {
    if (!mounted) {
      return;
    }
    context.read<B>().add(
          BasePagingEvent(
            queryData: queryData,
            limit: limit,
            isFirstRequest: true,
            needLoading: needLoading,
          ),
        );
  }

  void onLoadMore(
    Q? queryData,
    int limit, {
    bool needLoading = true,
  }) {
    if (!mounted) {
      return;
    }
    context.read<B>().add(
          BasePagingEvent(
            queryData: queryData,
            limit: limit,
            needLoading: needLoading,
          ),
        );
  }

  void onAdd(
    D newData, {
    bool focusOnUpdatedData = true,
  }) {
    if (!mounted) {
      return;
    }
    context.read<B>().add(
          AddToPagingDataEvent<D>(
            newData: newData,
            focusOnUpdatedData: focusOnUpdatedData,
          ),
        );
  }

  void onEdit(
    D updatedData,
    int currentPosition, {
    bool focusOnUpdatedData = true,
  }) {
    if (!mounted) {
      return;
    }
    context.read<B>().add(
          EditPagingDataEvent<D>(
            updatedData: updatedData,
            currentPosition: currentPosition,
            focusOnUpdatedData: focusOnUpdatedData,
          ),
        );
  }

  void onDelete(
    D data, {
    bool focusOnUpdatedData = false,
  }) {
    if (!mounted) {
      return;
    }
    context.read<B>().add(
          DeleteFromPagingDataEvent<D>(
            data: data,
            focusOnUpdatedData: focusOnUpdatedData,
          ),
        );
  }
}
